#ifndef _UTIL_H_
#define _UTIL_H_

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push,1)


/* physical memory interaction */
ULONG _System PhysMap(ULONG Length, ULONG Base, PVOID * Ptr);
ULONG _System PhysAlloc(ULONG Length, ULONG * Base, PVOID * Ptr);
ULONG _System PhysFree(PVOID Ptr);

/* pci config space interaction */
ULONG _System PciQueryVersion  (USHORT * version);
ULONG _System PciFindDevice    (USHORT VendorID, USHORT DeviceID, UCHAR Index, USHORT * addr);
ULONG _System PciFindClass     (ULONG ClassCode, UCHAR Index, USHORT * addr);
ULONG _System PciReadCfgDword  (USHORT addr, UCHAR reg, ULONG *data);
ULONG _System PciReadCfgByte   (USHORT addr, UCHAR reg, UCHAR *data);
ULONG _System PciWriteCfgDword (USHORT addr, UCHAR reg, ULONG data);
ULONG _System PciWriteCfgByte  (USHORT addr, UCHAR reg, UCHAR data);

/* some RTL replacements */
void memcpy(void * dst, void * src, int cnt);
#pragma aux memcpy =       \
        "rep movsb"        \
        parm [edi] [esi] [ecx] \
        modify [edi esi ecx]

/* shortest possible, non-blocking delay */
void _System  Delay(void);

/* Int10h support */
  typedef struct {
     BYTE bBufferType;
     BYTE bReserved;
     BYTE bSelCRF;           // CRF flag for the selector: ulong index into the CRF
     BYTE bOffCRF;           // CRF flag for the offset: ulong index into the CRF
     PVOID pAddress;         // linear address of the buffer
     ULONG ulSize;
  }BUFFER, *PBUFFER;

  /*
  ** defines for bBufferType field of BUFFER structure
  */
  #define BUFFER_NONE     0
  #define INPUT_BUFFER    1
  #define OUTPUT_BUFFER   2
  #define IO_BUFFER       (INPUT_BUFFER | OUTPUT_BUFFER)
  #define PASSED_DS_ES    0x04 // use DS/ES values as passed in

  /*
  **  image of CPU registers for BIOS calls
  */
  typedef struct vcrf_s {
     ULONG reg_eax;
     ULONG reg_ebx;
     ULONG reg_ecx;
     ULONG reg_edx;
     ULONG reg_ebp;
     ULONG reg_esi;
     ULONG reg_edi;
     ULONG reg_ds;
     ULONG reg_es;
     ULONG reg_fs;
     ULONG reg_gs;
     ULONG reg_cs;
     ULONG reg_eip;
     ULONG reg_eflag;
     ULONG reg_ss;
     ULONG reg_esp;
  }VCRF;

  #define OFFSET_EAX         0
  #define OFFSET_EBX         1
  #define OFFSET_ECX         2
  #define OFFSET_EDX         3
  #define OFFSET_EBP         4
  #define OFFSET_ESI         5
  #define OFFSET_EDI         6
  #define OFFSET_DS          7
  #define OFFSET_ES          8
  #define OFFSET_FS          9
  #define OFFSET_GS          10

  typedef struct {
     ULONG ulBIOSIntNo;      /* 0x10 for INT10 calls */
     VCRF aCRF;
     BUFFER pB[2];
  }INTCRF, * PINTCRF;

APIRET APIENTRY DosSysCtl(ULONG, PVOID);
#define  MINIVDM_QUERY          3
#define  MINIVDM_CREATE         4
#define  MINIVDM_DESTROY        5
#define  MINIVDM_INTERRUPT      6


#pragma pack(pop)

#ifdef __cplusplus
}
#endif

#endif
